/*
 * main.c
 * Copyright (c) 1994 by Christopher Heng. All rights reserved.
 * You may not remove or alter the copyright notice and/or the conditions for
 * use and distribution. See COPYING for additional conditions for use and
 * distribution. (The file COPYING contains the GNU General Public License
 * version 2.)
 *
 * Program to undigest a file comprising one or more digest files into
 * separate digests.
 *
 * Usage: splitdigest [options] [file...]
 * Options include
 *	-C <config>	Use configuration file specified.
 *	-c		Compress output file.
 *	-s		Separate the digests only; do not undigest.
 *	-t		Leave output file as uncompressed text file (default).
 *	-u		Undigest while separating digests (default).
 *	-h		Display usage.
 *	-v		Display version number.
 *
 * $Id: main.c,v 2.2 1994/05/09 02:41:29 chris Released $
 */

#include <getopt.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "cftables.h"
#include "splitdigest.h"
#include "version.h"

/* the following are for easy identification for bug reports and */
/* debugging purposes */
static char rcsid[] = "$Id: main.c,v 2.2 1994/05/09 02:41:29 chris Released $";
static char verstr[] = VERSTR;

int compression = DEFTOCOMPRESS ;	/* whether to compress or not */
#if (OLDMETHOD == 1)
int undigest = 0;
#else
int undigest = 1;	/* whether to undigest or not (by default) */
#endif
char * pgmname ;	/* program name */
char * configfile = DEFCONFIG;	/* configuration filename */
char * infilename = "stdin" ;

/* local functions */
static void cleanuponsig ( int sig );
static void cleanuponexit( void );
static int isdevice ( FILE * fp );

int main ( int argc, char ** argv )
{

	static char optstring[] = "C:chstuv" ;	/* options for getopt() */
	FILE * infile ;
	int i, c ;

	/* form pgmname without path prefix */
	if ((pgmname = strrchr( argv[0], '/' )) == NULL)
		pgmname = argv[0] ;
	else pgmname++;

	/* trap SIGINT so that we can clean up temporary files */
	if (signal( SIGINT, SIG_IGN ) != SIG_IGN) {
		signal( SIGINT, cleanuponsig );
	}
	/* register an atexit function */
	atexit( cleanuponexit );

	while ( (c = getopt(argc, argv, optstring)) != EOF ) {
		switch( c ) {
			case 'C':	/* specify another config file */
				configfile = optarg ;
				break;
			case 'c':	/* compress output file */
				compression = 1;
				break ;
			case 'h':	/* usage help requested */
				fprintf( stdout, "%s\n%s", verstr, USAGESTR );
				return EXIT_SUCCESS;
					/* quit immediately */
				break ;
			case 's':	/* separate digests only */
				undigest = 0;
				break;
			case 't':	/* switch off compression */
				compression = 0;
				break ;
			case 'u':	/* undigest */
				undigest = 1;
				break;
			case 'v':	/* version number request */
				fprintf( stdout, "%s\n", verstr );
				return EXIT_SUCCESS;
					/* quit immediately */
				break ;
			default:	/* bad option */
				fprintf( stderr, "\n%s\n", SHUSAGESTR);
				return EXIT_FAILURE;
				break ;
		}
	}
	if (inittables())
		return EXIT_FAILURE ;
	/* if no args assume stdin is the input */
	if (optind >= argc) {
		if (isdevice(stdin)) {
			fprintf( stderr, "\n%s\n", SHUSAGESTR );
			return EXIT_FAILURE ;
		}
		uncatfile( stdin );
	}
	else {	/* invoked uncatfile for each arg */
		for (i = optind; i < argc; i++) {
			infilename = argv[i];
			if ((infile = fopen ( infilename, READMODE ))==NULL) {
				fprintf( stderr, "%s: cannot open %s. "
					"Skipped.\n", pgmname, argv[i] );
				continue;
			}
			uncatfile( infile );
			fclose(infile);
		}
	}
	return EXIT_SUCCESS;

}

/* clean up on SIGINT */
static void cleanuponsig ( int sig )
{
	exit(EXIT_SIGINT) ;	/* the atexit() function will do the cleaning*/
}

/* atexit cleanup function */
static void cleanuponexit( void )
{
	realcleanup();
}

/* check if file pointer is a device */
static int isdevice ( FILE * fp )
{
	struct stat buf ;
	if (fstat( fileno(fp), &buf )) {
		fprintf( stderr, "%s: Error - can't stat stdin.\n");
		exit(EXIT_FAILURE);
	}
	return (S_ISCHR(buf.st_mode)) ;
}
